@@ -19,7 +19,7 @@ import threading
 import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('XApp', '1.0')
-from gi.repository import Gtk, Gdk, GdkPixbuf, GObject, Pango, XApp
+from gi.repository import Gtk, Gdk, GdkPixbuf, GLib, GObject, Pango, XApp
 
 from aptsources.sourceslist import SourcesList
 from io import BytesIO
@@ -208,7 +208,7 @@ def add_key_remote(key):
     return True
 
 def repo_malformed(line):
-    r = re.compile(r'(?:deb|deb-src)\s+\w+:/\S+?/?\s+\S+')
+    r = re.compile(r'(?:deb|deb-src)\s+(?:\[[^\]]+\]\s+)?\w+:/\S+?/?\s+\S+')
     match_line = r.match(line)
     if not match_line:
         return True
@@ -344,7 +344,7 @@ class Key():
         subprocess.call(["apt-key", "del", self.pub])
 
     def get_name(self):
-        return "%s\n<small>    %s</small>" % (GObject.markup_escape_text(self.uid), GObject.markup_escape_text(self.pub))
+        return "%s\n<small>    %s</small>" % (GLib.markup_escape_text(self.uid), GLib.markup_escape_text(self.pub))
 
 class Mirror():
     def __init__(self, country_code, url, name):
@@ -453,7 +453,7 @@ class ComponentSwitchBox(Gtk.Box):
         self.component = component
         self.window_object = window
         Gtk.Box.__init__(self)
-        label = Gtk.Label(self.component.description)
+        label = Gtk.Label(label=self.component.description)
         self.pack_start(label, False, False, 0)
         self.switch = Gtk.Switch()
         self.pack_end(self.switch, False, False, 0)
@@ -517,6 +517,7 @@ class MirrorSelectionDialog(object):
 
         r = Gtk.CellRendererPixbuf()
         col = Gtk.TreeViewColumn(_("Country"), r, pixbuf = MirrorSelectionDialog.MIRROR_COUNTRY_FLAG_COLUMN)
+        col.set_cell_data_func(r, self.data_func_surface)
         self._treeview.append_column(col)
         col.set_sort_column_id(MirrorSelectionDialog.MIRROR_TOOLTIP_COLUMN)
 
@@ -538,6 +539,11 @@ class MirrorSelectionDialog(object):
         with open('/usr/lib/linuxmint/mintSources/countries.json', encoding="utf-8", errors="ignore") as data_file:
             self.countries = json.load(data_file)
 
+    def data_func_surface(self, column, cell, model, iter_, *args):
+        pixbuf = model.get_value(iter_, MirrorSelectionDialog.MIRROR_COUNTRY_FLAG_COLUMN)
+        surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, self._application.scale)
+        cell.set_property("surface", surface)
+
     def _row_activated(self, treeview, path, view_column):
         self._dialog.response(Gtk.ResponseType.APPLY)
 
@@ -560,11 +566,12 @@ class MirrorSelectionDialog(object):
                 flag = FLAG_PATH % '_generic'
             tooltip = country_name
             if mirror.name != mirror.url:
-                tooltip = "%s: %s" % (country_name, mirror.name)
+                tooltip = "%s: %s" % (country_name, mirror.url)
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(flag, -1, FLAG_SIZE * self._application.scale)
             self._mirrors_model.append((
                 mirror,
                 mirror.url,
-                GdkPixbuf.Pixbuf.new_from_file_at_size(flag, -1, FLAG_SIZE),
+                pixbuf,
                 0,
                 None,
                 tooltip,
@@ -591,23 +598,31 @@ class MirrorSelectionDialog(object):
         except:
             return None
 
-    def check_mirror_up_to_date(self, url):
-        if (self.default_mirror_age is None or self.default_mirror_age < 2):
-            # If the default server was updated recently, the age is irrelevant (it would measure the time between now and the last update)
-            return True
+    def check_mirror_up_to_date(self, url, max_age):
         mirror_timestamp = self.get_url_last_modified(url)
         if mirror_timestamp is None:
             print ("Error: Can't find the age of %s !!" % url)
             return False
         mirror_date = datetime.datetime.fromtimestamp(mirror_timestamp)
         mirror_age = (self.default_mirror_date - mirror_date).days
-        if (mirror_age > 2):
+        # print("age: %d, max: %d - %s"%(mirror_age, max_age, url))
+        if (mirror_age > max_age):
             print ("Error: %s is out of date by %d days!" % (url, mirror_age))
             return False
         else:
             # Age is fine :)
             return True
 
+    def check_mint_mirror_up_to_date(self, url):
+        if (self.default_mirror_age is None or self.default_mirror_age < 2):
+            # print("Skipping Mint mirror check, < 2 days old: %s" % url)
+            # If the default server was updated recently, the age is irrelevant (it would measure the time between now and the last update)
+            return True
+        return self.check_mirror_up_to_date(url, 2)
+
+    def check_base_mirror_up_to_date(self, url):
+        return self.check_mirror_up_to_date(url, 14)
+
     @run_async
     def _all_speed_tests(self):
         model_iters = [] # Don't iterate through iters directly.. we're modifying their orders..
@@ -650,11 +665,16 @@ class MirrorSelectionDialog(object):
     def _speed_test(self, iter, url):
         download_speed = 0
         try:
+            up_to_date = False
+
             if self.is_base:
                 test_url = "%s/dists/%s/main/binary-amd64/Packages.gz" % (url, self.codename)
+                up_to_date = self.check_base_mirror_up_to_date("%s/ls-lR.gz" % url)
             else:
                 test_url = "%s/dists/%s/main/Contents-amd64.gz" % (url, self.codename)
-            if (self.is_base or self.check_mirror_up_to_date("%s/db/version" % url)):
+                up_to_date = self.check_mint_mirror_up_to_date("%s/db/version" % url)
+
+            if up_to_date:
                 c = pycurl.Curl()
                 buff = BytesIO()
                 c.setopt(pycurl.URL, test_url)
@@ -766,7 +786,12 @@ class MirrorSelectionDialog(object):
         # Try to find the age of the Mint archive
         self.default_mirror_age = None
         self.default_mirror_date = None
-        mirror_timestamp = self.get_url_last_modified("%s/db/version" % self.default_mirror)
+
+        if self.is_base:
+            mirror_timestamp = self.get_url_last_modified("%s/ls-lR.gz" % self.default_mirror)
+        else:
+            mirror_timestamp = self.get_url_last_modified("%s/db/version" % self.default_mirror)
+
         if mirror_timestamp is not None:
             self.default_mirror_date = datetime.datetime.fromtimestamp(mirror_timestamp)
             now = datetime.datetime.now()
@@ -795,8 +820,6 @@ class Application(object):
         self._interface_loaded = False
         self._currently_applying_sources = False
 
-        self.infobar_visible = False
-
         self.lsb_codename = subprocess.getoutput("lsb_release -sc")
 
         glade_file = "/usr/lib/linuxmint/mintSources/mintsources.glade"
@@ -805,11 +828,15 @@ class Application(object):
         self.builder.set_translation_domain("mintsources")
         self.builder.add_from_file(glade_file)
         self._main_window = self.builder.get_object("main_window")
+        self._info_box = self.builder.get_object("box_infobar")
+        self._info_revealer = self.builder.get_object("info_revealer")
 
         self._main_window.set_title(_("Software Sources"))
 
         self._main_window.set_icon_name("software-sources")
 
+        self.scale = self._main_window.get_scale_factor()
+
         self._official_repositories_page = self.builder.get_object("official_repositories_page")
 
         self.apt = mintcommon.aptdaemon.APT(self._main_window)
@@ -1052,7 +1079,7 @@ class Application(object):
                             mirror = Mirror(country_code, url, name)
                             mirror_list.append(mirror)
         if path.endswith("Debian.mirrors"):
-            mirror = Mirror("WD", "http://deb.debian.org/debian", "http://deb.debian.org/debian/")
+            mirror = Mirror("WD", "https://deb.debian.org/debian", "https://deb.debian.org/debian/")
             mirror_list.append(mirror)
         return mirror_list
 
@@ -1680,27 +1707,28 @@ class Application(object):
 
 
     def enable_reload_button(self):
-        if not self.infobar_visible:
-            self.infobar_visible = True
-            infobar = Gtk.InfoBar()
-            infobar.set_message_type(Gtk.MessageType.INFO)
-            box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)
-            image = Gtk.Image.new_from_icon_name("dialog-information-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
-            box.pack_start(image, False, False, 0)
-            info_label = Gtk.Label()
-            infobar_message = "<b>%s</b>\n%s" % (_("Your configuration changed"), _("Click OK to update your APT cache"))
-            info_label.set_markup(infobar_message)
-            box.pack_start(info_label, False, False, 0)
-            infobar.get_content_area().pack_start(box, False, False, 0)
-            infobar.add_button(_("OK"), Gtk.ResponseType.OK)
-            infobar.add_button(_("Cancel"), Gtk.ResponseType.CANCEL)
-            infobar.connect("response", self._on_infobar_response)
-            self.builder.get_object("box_infobar").pack_start(infobar, True, True, 5)
-            infobar.show_all()
+        if not self._info_revealer.get_reveal_child():
+            if self._info_box.get_children() == []:
+                infobar = Gtk.InfoBar()
+                infobar.set_message_type(Gtk.MessageType.INFO)
+                box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)
+                image = Gtk.Image.new_from_icon_name("dialog-information-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
+                box.pack_start(image, False, False, 0)
+                info_label = Gtk.Label()
+                infobar_message = "<b>%s</b>\n%s" % (_("Your configuration changed"), _("Click OK to update your APT cache"))
+                info_label.set_markup(infobar_message)
+                box.pack_start(info_label, False, False, 0)
+                infobar.get_content_area().pack_start(box, False, False, 0)
+                infobar.add_button(_("OK"), Gtk.ResponseType.OK)
+                infobar.add_button(_("Cancel"), Gtk.ResponseType.CANCEL)
+                infobar.connect("response", self._on_infobar_response)
+                infobar.show_all()
+                self._info_box.pack_start(infobar, True, True, 0)
+
+            self._info_revealer.set_reveal_child(True)
 
     def _on_infobar_response(self, infobar, response_id):
-        infobar.destroy()
-        self.infobar_visible = False
+        self._info_revealer.set_reveal_child(False)
 
         if response_id == Gtk.ResponseType.OK:
             self.apt.update_cache()
@@ -1845,11 +1873,13 @@ class Application(object):
                     base_flag_path = flag
                 break
 
-        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(mint_flag_path, -1, FLAG_SIZE)
-        self.builder.get_object("image_mirror").set_from_pixbuf(pixbuf)
+        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(mint_flag_path, -1, FLAG_SIZE * self.scale)
+        surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, self.scale)
+        self.builder.get_object("image_mirror").set_from_surface(surface)
 
-        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(base_flag_path, -1, FLAG_SIZE)
-        self.builder.get_object("image_base_mirror").set_from_pixbuf(pixbuf)
+        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(base_flag_path, -1, FLAG_SIZE * self.scale)
+        surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, self.scale)
+        self.builder.get_object("image_base_mirror").set_from_surface(surface)
 
     def get_clipboard_text(self, source_type):
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
